﻿
using System.Collections.Generic;

using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;
namespace LittleBigTable
{
    struct VertexPositionTexture
    {
        public Vector4 Position;
        public Vector2 TexCoord;
        public static readonly InputElement[] inputElements = new[] {
				new InputElement("POSITION", 0, Format.R32G32B32A32_Float, 0, 0),
				new InputElement("TEXCOORD",0,Format.R32G32_Float, 16 ,0)
			};
        public static readonly int SizeInBytes = System.Runtime.InteropServices.Marshal.SizeOf(typeof(VertexPositionTexture));
        public VertexPositionTexture(Vector4 position, Vector2 texCoord)
        {
            Position = position;
            TexCoord = texCoord;
        }
        public VertexPositionTexture(Vector3 position, Vector2 texCoord)
        {
            Position = new Vector4(position, 1);
            TexCoord = texCoord;
        }
    }
    //Use TextSets to control the strings 
    public class TextSets
    {
        protected Texture2D TextureD3D11;
        KeyedMutex MutexD3D10;
        KeyedMutex MutexD3D11;
        protected SlimDX.Direct2D.RenderTarget dwRenderTarget;
        List<Text> Texts;
    
        SlimDX.Direct3D10_1.Device1 device10_1;
        SlimDX.Direct2D.Factory d2Factory;
        int TextCount;
        SlimDX.DirectWrite.Factory dwFactory;
        public void SetPositionByIndex(Vector3 position,int index)
         {
             Texts[index].worldPosition = position;
         }
        public TextSets(Device device11,Adapter1 adapter1,Effect effect,Camera camera)
        {
            Texts = new List<Text>();
            dwFactory = new SlimDX.DirectWrite.Factory(
                SlimDX.DirectWrite.FactoryType.Isolated
            );
            device10_1 = new SlimDX.Direct3D10_1.Device1(
             adapter1,
             SlimDX.Direct3D10.DriverType.Hardware,
             SlimDX.Direct3D10.DeviceCreationFlags.BgraSupport | SlimDX.Direct3D10.DeviceCreationFlags.Debug,
             SlimDX.Direct3D10_1.FeatureLevel.Level_10_0
             );

            TextureD3D11 = new Texture2D(device11, new Texture2DDescription
            {
                Width = (int)camera.GetWidth(),
                Height = (int)camera.GetHeight(),
                MipLevels = 1,
                ArraySize = 1,
                Format = Format.B8G8R8A8_UNorm,
                SampleDescription = new SampleDescription(1, 0),
                Usage = ResourceUsage.Default,
                BindFlags = BindFlags.RenderTarget | BindFlags.ShaderResource,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.KeyedMutex
            });

            SlimDX.DXGI.Resource sharedResource = new SlimDX.DXGI.Resource(TextureD3D11);
            SlimDX.Direct3D10.Texture2D textureD3D10 = device10_1.OpenSharedResource<SlimDX.Direct3D10.Texture2D>(sharedResource.SharedHandle);
            MutexD3D10 = new KeyedMutex(textureD3D10);
            MutexD3D11 = new KeyedMutex(TextureD3D11);
            d2Factory = new SlimDX.Direct2D.Factory(
               SlimDX.Direct2D.FactoryType.SingleThreaded,
               SlimDX.Direct2D.DebugLevel.Information
           );
            // Query for a IDXGISurface.
            // DirectWrite and DirectX10 can interoperate thru DXGI.
            Surface surface = textureD3D10.AsSurface();
            SlimDX.Direct2D.RenderTargetProperties rtp = new SlimDX.Direct2D.RenderTargetProperties();
            rtp.MinimumFeatureLevel = SlimDX.Direct2D.FeatureLevel.Direct3D10;
            rtp.Type = SlimDX.Direct2D.RenderTargetType.Hardware;
            rtp.Usage = SlimDX.Direct2D.RenderTargetUsage.None;
            rtp.PixelFormat = new SlimDX.Direct2D.PixelFormat(Format.Unknown, SlimDX.Direct2D.AlphaMode.Premultiplied);
            dwRenderTarget = SlimDX.Direct2D.RenderTarget.FromDXGI(d2Factory, surface, rtp);          
          
        }
        public void ResizeTexture(int width, int height, Device device11, Adapter1 adapter1)
        {
            TextureD3D11.Dispose();
            MutexD3D10.Dispose();
            MutexD3D11.Dispose();
            TextureD3D11 = new Texture2D(device11, new Texture2DDescription
            {
                Width = width,
                Height = height,
                MipLevels = 1,
                ArraySize = 1,
                Format = Format.B8G8R8A8_UNorm,
                SampleDescription = new SampleDescription(1, 0),
                Usage = ResourceUsage.Default,
                BindFlags = BindFlags.RenderTarget | BindFlags.ShaderResource,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.KeyedMutex
            });
            SlimDX.DXGI.Resource sharedResource = new SlimDX.DXGI.Resource(TextureD3D11);
            SlimDX.Direct3D10.Texture2D textureD3D10 = device10_1.OpenSharedResource<SlimDX.Direct3D10.Texture2D>(sharedResource.SharedHandle);
            MutexD3D10 = new KeyedMutex(textureD3D10);
            MutexD3D11 = new KeyedMutex(TextureD3D11);
            // Query for a IDXGISurface.
            // DirectWrite and DirectX10 can interoperate thru DXGI.
            Surface surface = textureD3D10.AsSurface();
            SlimDX.Direct2D.RenderTargetProperties rtp = new SlimDX.Direct2D.RenderTargetProperties();
            rtp.MinimumFeatureLevel = SlimDX.Direct2D.FeatureLevel.Direct3D10;
            rtp.Type = SlimDX.Direct2D.RenderTargetType.Hardware;
            rtp.Usage = SlimDX.Direct2D.RenderTargetUsage.None;
            rtp.PixelFormat = new SlimDX.Direct2D.PixelFormat(Format.Unknown, SlimDX.Direct2D.AlphaMode.Premultiplied);
            dwRenderTarget = SlimDX.Direct2D.RenderTarget.FromDXGI(d2Factory, surface, rtp);
        }
        public void ReleaseTexture()
        {
            TextureD3D11.Dispose();        
        }
        public void AddTexts(Text text)
        {
            text.InitializeText(dwRenderTarget, dwFactory);
            Texts.Add(text);
            TextCount++;
        }
        public void DeleteText(Text text)
        {
            if (text != null && text.IsCreated)
            {
                Texts.Remove(text);
            }
         
        }
        public void drawTexts(Camera camera,Device device11,Effect effect)
        {
            MutexD3D10.Acquire(0, 100);
            dwRenderTarget.BeginDraw();
            dwRenderTarget.Clear(new Color4(0, 0, 0, 0));
            //Draw each text 
            for (int i = 0; i < Texts.Count; i++)
            {
                Texts[i].ApplyEffect();
                Texts[i].drawText(camera, dwRenderTarget);
            }
            dwRenderTarget.EndDraw();
            MutexD3D10.Release(0);
            // Draw the shared texture2D onto the screen
            // Need to Aquire the shared texture for use with DirectX11
            MutexD3D11.Acquire(0, 100);
            ShaderResourceView srv = new ShaderResourceView(device11, TextureD3D11);
            InitializeGeometry(device11);
            
            effect.GetVariableByName("g_textOverlay").AsResource().SetResource(srv);
            effect.GetTechniqueByName("Text").GetPassByName("P0").Apply(device11.ImmediateContext);
            
            device11.ImmediateContext.Draw(6, 0);
            srv.Dispose();
            MutexD3D11.Release(0);
           
        }
        protected void InitializeGeometry(Device device)
        {
            
            float[] vertices = new[]
                                    {  
                                            -1.0f,  1.0f, -0.2f, 0.0f, 0.0f , 0.0f , 0.0f,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                             1.0f, 1.0f ,-0.2f,  1.0f, 0.0f , 0.0f  ,0.0f,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                            -1.0f, -1.0f ,-0.2f ,0.0f ,1.0f , 0.0f , 0.0f, -1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                            -1.0f ,-1.0f ,-0.2f ,0.0f, 1.0f , 0.0f , 0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                             1.0f , 1.0f, -0.2f ,1.0f ,0.0f , 0.0f , 0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f,
                                             1.0f,  -1.0f, -0.2f ,1.0f, 1.0f , 0.0f  ,0.0f ,-1.0f,1.0f,0.0f,0.0f,0.0f,-1.0f,0.0f
                                    };
       

            short[] faces = new[]
                                {
                                        (short)0, (short)1, (short)2,
                                        (short)3, (short)4, (short)5
                                };

            // Creating vertex buffer
            const int vertexSize = sizeof(float) * 14;
            var stream = new DataStream(6 * vertexSize, true, true);
            stream.WriteRange(vertices);
            stream.Position = 0;

            var vertexBuffer = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.VertexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Index buffer
            stream = new DataStream(6 * 2, true, true);
            stream.WriteRange(faces);
            stream.Position = 0;

            var indices = new SlimDX.Direct3D11.Buffer(device, stream, new BufferDescription
            {
                BindFlags = BindFlags.IndexBuffer,
                CpuAccessFlags = CpuAccessFlags.None,
                OptionFlags = ResourceOptionFlags.None,
                SizeInBytes = (int)stream.Length,
                Usage = ResourceUsage.Default
            });
            stream.Dispose();

            // Uploading to the device
            device.ImmediateContext.InputAssembler.PrimitiveTopology = PrimitiveTopology.TriangleList;
            device.ImmediateContext.InputAssembler.SetVertexBuffers(0, new VertexBufferBinding(vertexBuffer, vertexSize, 0));
            device.ImmediateContext.InputAssembler.SetIndexBuffer(indices, Format.R16_UInt, 0);
        }
    }
}
