﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using FarseerPhysics.Collision;
using FarseerPhysics.Common;
using FarseerPhysics.Controllers;
using FarseerPhysics.Dynamics;
using FarseerPhysics.Factories;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;
namespace LittleBigTable 
{
    using Vec2 = Microsoft.Xna.Framework.Vector2;   
    public class CircleObject :  Object
    {
        public float Radius;
       
        public void InitializeParameters(Device device, Shader shader, World world,InitializeInf objInf, float radius,TextureManager textureMgr)
        {
            
            ObjInf = new InitializeInf();
            ObjInf.CreateObjectInf(objInf.Position, objInf.ShadowPosition, objInf.ObjectDepth, objInf.ShadowDepth,
                objInf.Alpha, objInf.BlurExtended, objInf.Index, objInf.Pickable, objInf.BumpDensity, objInf.Density, objInf.TextLocalPosition);
            ObjInf.colorTextureFileName = objInf.colorTextureFileName;
            ObjInf.normalTextureFileName=objInf.normalTextureFileName;
            ObjInf.ShadowDarkness = objInf.ShadowDarkness;
            Radius = radius;
            ObjInf.Scale = new Vector2(radius, radius);
            SetTextures(device, shader, objInf.colorTextureFileName, objInf.normalTextureFileName, textureMgr);
            InitializeGeometry(device);
            if (IsPhysics)
            {
                InitializePhysicsBody(world);
            }

        }
        public CircleObject()
        {
            IsPhysics = true;
            IsDisposed = false;
            TotalHitBurden = 3000.0f;
            IsDrawShadow = true;
        }
        private void InitializePhysicsBody(World world)
        {
            //Initialize Physics Body 
            Body = BodyFactory.CreateCircle(world, Radius, ObjInf.Density);        
            Body.Position = new Vec2(ObjInf.Position.X, ObjInf.Position.Y);
            IsPhysics = true;
            Body.IsStatic = false;
            Body.Enabled = true;
            Body.LinearDamping = 0.2f;
            Body.AngularDamping = 0.2f;
            Body.Friction = 0.2f;
            Body.Restitution = 0.2f;

            initializeMouseJoint(world);
        }
        private void InitializeRenderBody(Device device)
        {
            //For a circle, we only need to create a rectangle
            InitializeGeometry(device);
        }
        public override void  Render(Shader shader,Device device,Lighting lighting, Camera camera,float castBoardZ)
        {

            float ObjToBackBoard = Math.Abs(castBoardZ- ObjInf.ObjectDepth);
            float LightToBoard = Math.Abs(lighting.GetLightPosition().Z - castBoardZ);
            float ObjToLightDown = Math.Abs(lighting.GetLightPosition().Z - ObjInf.ObjectDepth);
            float ObjToLight = (lighting.GetLightPosition() - GetPosition()).Length();
            float Ratio=Math.Abs(LightToBoard-ObjToBackBoard)/LightToBoard;
            float xFinal, yFinal;
            Vector3 positionObject;
            Vector3 positionShadow;
            if (IsPhysics)
            {
                xFinal = Body.Position.X / Ratio + lighting.GetLightPosition().X * (1 - 1 / Ratio);
                yFinal = Body.Position.Y / Ratio + lighting.GetLightPosition().Y * (1 - 1 / Ratio);
                positionObject = new Vector3(Body.Position.X, Body.Position.Y, ObjInf.ObjectDepth);
                positionShadow = new Vector3(xFinal, yFinal, ObjInf.ObjectDepth + 0.1f);
            }
            else
            {
                xFinal=ObjInf.Position.X/ Ratio + lighting.GetLightPosition().X * (1 - 1 / Ratio);
                yFinal = ObjInf.Position.Y / Ratio + lighting.GetLightPosition().Y * (1 - 1 / Ratio);
                positionObject = new Vector3(ObjInf.Position.X, ObjInf.Position.Y, ObjInf.ObjectDepth);
                positionShadow = new Vector3(xFinal, yFinal, ObjInf.ObjectDepth+0.1f);
            }
            
            Transform transform;
            float Angle;
            if (IsPhysics)
            {
                Body.GetTransform(out transform);
                Angle=transform.Angle;
                ObjInf.Position = new Vector2(Body.Position.X, Body.Position.Y);
                ObjInf.Angle = Body.Rotation; 
            }
            else
            {
                Angle=ObjInf.Angle;
            }
            //Set Textures to the shader 

           // SetTextures(device, shader, ObjInf.colorTextureFileName, ObjInf.normalTextureFileName);
            //Set data of circle to the shader
           // InitializeGeometry(device);

            shader.GetEffect().GetVariableByName("colorTexture").AsResource().SetResource(colorTextureRV);
            shader.GetEffect().GetVariableByName("normalTexture").AsResource().SetResource(normalTextureRV);
            if (IsDrawShadow)
            {
                shader.SetCircleShadow(device, Radius / Ratio, positionShadow, Angle, ObjInf.BlurExtended, ObjInf.ShadowDarkness);
                shader.Render(device, lighting, camera, 6);
            }
        
            shader.SetCircle(device, Radius, positionObject, Angle, ObjInf.Alpha,ObjInf.BumpDensity);
           // 
            shader.Render(device, lighting, camera, 6);
        }
        //Clear all the stuff of the circle object
        public void Dispose()
        {

        }
        public void Explosion(float maxRadius, float expandRate, float disappearRate)
        {
            Body.Enabled = false;
            SetPickable(false);
            if (Radius < maxRadius)
            {
                Radius += expandRate;
            }
            else
            {
                if (ObjInf.Alpha > 0.0f)
                {
                    ObjInf.Alpha -= disappearRate;
                }
                if (ObjInf.ShadowDarkness > 0.0f)
                {
                    ObjInf.ShadowDarkness -= disappearRate;
                }
            }
        }
    }
}
