﻿using System;
using System.Windows;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SlimDX;
using SlimDX.D3DCompiler;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Resource = SlimDX.Direct3D11.Resource;

using FarseerPhysics.Collision;
using FarseerPhysics.Common;
using FarseerPhysics.Controllers;
using FarseerPhysics.Dynamics;
using FarseerPhysics.Factories;

namespace LittleBigTable
{
    using Vec2 = Microsoft.Xna.Framework.Vector2;
    public class CollisionPairIndices
    { 
        public int index1;
        public int index2;
        public Vector3 p1;
        public Vector3 p2;
        public ZD3DObject a;
        public ZD3DObject b;
        public CollisionPairIndices(int id1,int id2, ZD3DObject a1, ZD3DObject b1)
        {
            index1=id1;
            index2=id2;
            a = a1;
            b = b1;

        }

        public Vector3 SupportFunction(Vector3 Direction)
        {
            Vector3 ak = a.SupportPoint(-Direction);
            p1 = ak;
            Vector3 bk= b.SupportPoint(Direction);
            p2 = bk;
            return ( bk-ak );
        }
    }
    public class Broadphase
    {
        List<sValue> listSEX;
        List<sValue> listSEY;
         List<sValue> listSEZ   ;
        //List<SAPCollisionPair> listOverlapping;
        List<int>[] indexInvX;
        List<int>[] indexInvY;
        List<int>[] indexInvZ;
        public List<int>[] indexInv;
        public void Update(List<ZD3DObject> ObjList)
        {
           
           // listOverlapping = new List<SAPCollisionPair>();
            for (int i = 0; i < ObjList.Count;i++ )
            {
                listSEX.Add(new sValue(ObjList[i].Position.X - ObjList[i].Radius, i, true));
                listSEY.Add(new sValue(ObjList[i].Position.Y - ObjList[i].Radius, i, true));
                listSEZ.Add(new sValue(ObjList[i].Position.Z - ObjList[i].Radius, i, true));


                listSEX.Add(new sValue(ObjList[i].Position.X + ObjList[i].Radius, i, false));
                listSEY.Add(new sValue(ObjList[i].Position.Y + ObjList[i].Radius, i, false));
                listSEZ.Add(new sValue(ObjList[i].Position.Z + ObjList[i].Radius, i, false));
            }
            listSEX.Sort(delegate(sValue p1, sValue p2) { return p1.value.CompareTo(p2.value); });
            listSEY.Sort(delegate(sValue p1, sValue p2) { return p1.value.CompareTo(p2.value); });
            listSEZ.Sort(delegate(sValue p1, sValue p2) { return p1.value.CompareTo(p2.value); });


            ExtractOverlapping(listSEX, ObjList, indexInvX);
            ExtractOverlapping(listSEY, ObjList, indexInvY);
            ExtractOverlapping(listSEZ, ObjList, indexInvZ);

            listSEX.Clear();
            listSEY.Clear();
            listSEZ.Clear();
            IntegradeXYZ(ObjList.Count);
            for(int i=0;i<ObjList.Count;i++)
            {
                 indexInvX[i].Clear();
                 indexInvY[i].Clear();
                 indexInvZ[i].Clear();
            }       
        }
        public Broadphase(List<ZD3DObject> ObjList)
        {
            listSEX = new List<sValue>();
            listSEY = new List<sValue>();
            listSEZ = new List<sValue>();

            indexInvX = new List<int>[ObjList.Count];
            indexInvY = new List<int>[ObjList.Count];
            indexInvZ = new List<int>[ObjList.Count];

            indexInv = new List<int>[ObjList.Count];

            for (int i = 0; i < indexInv.Length; i++)
            {
                indexInv[i] = new List<int>();
            }
        }
        public void IntegradeXYZ(int Count)
        {
            int[] Set=new int[Count];
            for(int i=0;i<Count;i++)
            {
                Set[i]=0;//All is zero
            }
            for (int k = 0; k < Count; k++)
            {
                for (int i = 0; i < indexInvX[k].Count; i++)
                {
                    Set[indexInvX[k][i]]++;
                }
                for (int i = 0; i < indexInvY[k].Count; i++)
                {
                    Set[indexInvY[k][i]]++;
                }
                for (int i = 0; i < indexInvZ[k].Count; i++)
                {
                    Set[indexInvZ[k][i]]++;
                }
                for (int i = 0; i < Count; i++)
                {
                    if (Set[i] == 3)
                    {
                        indexInv[k].Add(i);
                    }
                }
                for (int i = 0; i < Count; i++)
                {
                    Set[i] = 0;//All is zero
                }
            }

        }
       
        public void ExtractOverlapping(List<sValue> list, List<ZD3DObject> ObjList,List<int> []listIndex)
        {
            List<int> Index;
           
            Index = new List<int>();
            
            for (int i = 0; i < ObjList.Count; i++)
            {
                listIndex[i] = new List<int>();
            }
            for (int i = 0; i < list.Count; i++)
                {
                    if (list[i].isS == true)
                    {
                        for (int j = 0; j < Index.Count; j++)
                        {
                           // listOverlapping.Add(new SAPCollisionPair(Index[j], listSE[i].index));
                            if (Index[j] > list[i].index)
                            {
                                listIndex[Index[j]].Add(list[i].index);
                            }
                            else
                            {
                                listIndex[list[i].index].Add(Index[j]);
                            }
                            
                        }
                        Index.Add(list[i].index);
                    }
                    else
                    {
                        Index.Remove(list[i].index);
                    }

                }
        }
        public void Clear(int Count)
        {
            for (int i = 0; i < Count; i++)
            {
                indexInv[i].Clear();
            }     
            //listOverlapping.Clear();
        }
    }
    public class SAPCollisionPair
    {
        public int index1;
        public int index2;
        public SAPCollisionPair(int idx1,int idx2)
        {
            index1 = idx1;
            index2 = idx2;
        }
        
    }
    public class sValue
    {
        public float value;
       
        public int index;
        public bool isS;
       public  sValue(float v,int idx,bool S)
        {
            value = v;
            index = idx;
            isS = S;
        }
      
    }

    public enum SimulationStyle { Euler, RK2, RK4 }
    public class SceneManager
    {
        public bool OnRendering;
        DateTime d;
        int FrameCount;
        long Time;
        public TextureManager TextureMgr { get; set; }
        public SimulationStyle SceneSimulationStyle;
        public Spring SpringConstraint;
        Camera SceneCamera;
        D3DDevice SceneDevice;
        ObjectSets SceneObjSets;
        TextSets TSets;
        Shader SceneShader;
        public ZD3DObject SpringMass;
        World SceneWorld;
        Surface surface;
        System.DateTime PreviousTime;
        Object PickingObj;
        double TimeInterval;
        Lighting lighting;
        CircleObject circle2;
        public float Powder;
        public float Angle;
        public float Mass;
        Vector3 PickingPosition;
        public int[] MousePosition;
        System.TimeSpan interval;
        IntPtr Handle;
        Broadphase broadphase;
        float avryInterval;
        long fps;
        int Width;
        int Height;
        ZD3DObject a;
        ZD3DObject Ground;
        ZD3DObject Ground1;
        ZD3DObject Ground2;
        ZD3DObject Ground3;
        ZD3DObject Ground4;
        ZD3DObject Upper;
        ZD3DObject b;
        Bunny bc;
        ZD3DObject bunny;
        ZD3DObject bunny2;
        GJK gjk;
        public  ZD3DObject Bullet;
        ZD3DObject Base;
        List<CollisionPairIndices> CollisionPair;
        public List<ZD3DObject> CollisionObj;

        List<ZD3DObject> Gunlist;
        List<ZD3DObject> SpringList;
        public bool Pressed { get; set; }
        
        protected SlimDX.Direct2D.RenderTarget dwRenderTarget;
        public SceneManager(IntPtr handle, int width, int height)
        {
            Handle = handle;
            Width = width;
            Height = height;
        


            Width = width;
            Height = height;
            Initialize();
            SceneSimulationStyle = SimulationStyle.Euler;
            gjk = new GJK();
        }
        public void InitializeCollisionPair()
        {
          

        }
        public void AddCollisionObj(ZD3DObject Obj)
        {
            
            for(int i=0;i<CollisionObj.Count;i++)
            {
                CollisionPairIndices p=new CollisionPairIndices(i,CollisionObj.Count,CollisionObj[i],Obj);
                CollisionPair.Add(p);
            }
            CollisionObj.Add(Obj);
        }
        public void CollisionDetection()
        {
            int Count = 0;
            for (int i = 0; i < CollisionPair.Count; i++)
            {
                if (CollisionObj[CollisionPair[i].index1].CollisionDetectionObj2Obj(CollisionObj[CollisionPair[i].index2]))
                {
                    Count++;
                    if (gjk.Intersects(CollisionPair[i].SupportFunction))
                    {
                      
                        Vector3 distance = CollisionPair[i].p1 - CollisionPair[i].p2;
                        Vector3 normal = CollisionPair[i].a.WorldVertics[CollisionPair[i].a.indexSplit].normal;
                        normal.Normalize();
                        distance = Vector3.Dot(distance, normal) * normal * 0.5f;

                        CollisionPair[i].a.Position += distance;
                        CollisionPair[i].b.Position -= distance;
                        CollisionPair[i].a.Velocity = -CollisionPair[i].a.Velocity;
                        CollisionPair[i].a.AngularVelocity = -CollisionPair[i].a.AngularVelocity;
                        CollisionPair[i].b.Velocity = -CollisionPair[i].b.Velocity;
                        CollisionPair[i].b.AngularVelocity = -CollisionPair[i].b.AngularVelocity;
                    }

                }
            }
        }
        //public void DeleteObj(int index)
        //{
        //    for(int i=0;i<CollisionObj.Count;i++)
        //    {
        //        if()
        //    }
        //}
        public int CountRendered()
        {
            int Count=0;
            for (int i = 0; i < SceneObjSets.GetObjectCount(); i++)
            {
                if (SceneObjSets.GetObjectByIndex(i).WillBeRendered == true)
                {
                    Count++;
                }
            }
            return Count;
        }
        public void SetMousePosition(int x,int y)
        {
            MousePosition = new int[2];
            MousePosition[0] = x;
            MousePosition[1] = y;
        }
        public Lighting GetLighting()
        {
            return lighting;
        }
        public ObjectSets GetObjectSets()
        {
            return SceneObjSets;
        }
        public Shader GetShader()
        {
            return SceneShader;
        }
        public void CreateBlocksAroungd(Vector2 center,float width,float height)
        {
            Body Right = BodyFactory.CreateRectangle(SceneWorld, 16.0f, height, 0);
            Right.Position = new Vec2(center.X + width / 2 + 8.0f, center.Y);
            Right.IsStatic = true;
            Right.Enabled = true;
            Right.Friction = 1.0f;

            Body Left = BodyFactory.CreateRectangle(SceneWorld,16.0f, height, 0);
            Left.Position = new Vec2(center.X - width / 2 - 8.0f, center.Y);
            Left.IsStatic = true;
            Left.Enabled = true;
            Left.Friction = 1.0f;

            Body Top = BodyFactory.CreateRectangle(SceneWorld, width, 16.0f, 0);
            Top.Position = new Vec2(center.X, center.Y + height / 2 + 8.0f);
            Top.IsStatic = true;
            Top.Enabled = true;
            Top.Friction = 1.0f;

            Body Bottom = BodyFactory.CreateRectangle(SceneWorld, width, 16.0f, 0);
            Bottom.Position = new Vec2(center.X, center.Y - height / 2-8.0f);
            Bottom.IsStatic = true;
            Bottom.Enabled = true;
            Bottom.Friction = 1.0f;
        }
        public Vector2 GetPickingPositionFromBackGround()
        {
            Vector3 v3=new Vector3();
            if (circle2.Pick(MousePosition[0], MousePosition[1], out v3, SceneCamera))
            {
                return new Vector2(v3.X, v3.Y);
            }
            else
            {
                return new Vector2(0, 0);
            }
        }

        public Vector2 GetWorldPositionFromScreen(float x, float y)
        {
            Vector3 v3 = new Vector3();
            if (circle2.Pick(x, y, out v3, SceneCamera))
            {
                return new Vector2(v3.X, v3.Y);
            }
            else
            {
                return new Vector2(0, 0);
            }
        }
        public void Confine()
        {
            foreach (ZD3DObject o in CollisionObj)
            {
                o.EularSover(0.01f);
                float Bound = 50.0f;
                if (o.Position.X < -Bound)
                {
                    o.Position.X += -Bound - o.Position.X;
                    o.Velocity = -o.Velocity;
                    o.AngularVelocity = -o.AngularVelocity;
                }
                if (o.Position.Y < -Bound)
                {
                    o.Velocity = -o.Velocity;
                    o.AngularVelocity = -o.AngularVelocity;
                    o.Position.Y += -Bound - o.Position.Y;
                }
                if (o.Position.Z < -Bound)
                {
                    o.AngularVelocity = -o.AngularVelocity;
                    o.Position.Z += -Bound - o.Position.Z;
                }
                if (o.Position.X > Bound)
                {
                    o.Velocity = -o.Velocity;
                    o.AngularVelocity = -o.AngularVelocity;
                    o.Position.X += Bound - o.Position.X;
                }
                if (o.Position.Y > Bound)
                {
                    o.Velocity = -o.Velocity;
                    o.AngularVelocity = -o.AngularVelocity;
                    o.Position.Y += Bound - o.Position.Y;
                }
                if (o.Position.Z > Bound)
                {
                    o.Velocity = -o.Velocity;
                    o.AngularVelocity = -o.AngularVelocity;
                    o.Position.Z += Bound - o.Position.Z;
                }
            }
        }
        public void Initialize()
        {
            d = new DateTime();
            FrameCount = 0;
            Time = 0;
            OnRendering = false;
            SceneCamera = new Camera();
            SceneCamera.InitializeCamera(new Vector3(60.0f, 60.0f, 60.0f), new Vector3(-50, 0.0f, -100.0f), new Vector3(0.0f, 0.0f, 1.0f),
                0.8f, Width, Height, 0.1f, 4000.0f);
            SceneDevice = new D3DDevice();
            SceneDevice.CreateDevice(Handle,Width, Height);
            SceneObjSets = new ObjectSets();
            SceneShader = new Shader();
            SceneShader.CompileShader(SceneDevice.GetDevice(), "Effet.fx", "fx_5_0");
            SceneWorld = new World(new Vec2(0.0f, -30.0f));
            TextureMgr = new TextureManager();
            lighting = new Lighting();
            lighting.SetLightPosition(new Vector3(0.0f,40.0f,-30.0f));
            lighting.SetAmbientValue(0.4f);
            lighting.SetSpotIntensy(2.0f);
            CollisionObj = new List<ZD3DObject>();
            CollisionPair = new List<CollisionPairIndices>();

           
            
            bc=new Bunny();
            Model1 md1 = new Model1();
            Model2 md2 = new Model2();
            
            //bunny = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Blue.jpg", "glazed_brick_N.jpg", TextureMgr);
            //bunny.Position = new Vector3(0.0f,12.2f,6.0f);
            //bunny.Scale = new Vector3(10.0f, 10.0f, 5.0f);
            //bunny.SetGeometry(md2.vertices, md2.faces, SceneDevice.GetDevice(), md2.PositionFace);
            //bunny.SetRotation(3.14f/4,new Vector3(0.0f,0.0f,1.0f));



            //AddCollisionObj(bunny);


            //bunny2 = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Blue.jpg", "glazed_brick_N.jpg", TextureMgr);
            //bunny2.Position = new Vector3(0.0f, 12.2f, -20.0f);
            //bunny2.Velocity = new Vector3(0.0f, 0.0f, 0.01f);
            //bunny2.Scale = new Vector3(5.0f, 5.0f, 5.0f);
            //bunny2.SetGeometry(bc.vertices, bc.faces, SceneDevice.GetDevice(), bc.PositionFace);

           
            //AddCollisionObj(bunny2);


            


            Random r=new Random();

            for (int i = 0; i < 2; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    for (int k = 0; k < 2; k++)
                    {
                        ZD3DObject car = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Blue.jpg", "glazed_brick_N.jpg", TextureMgr);
                        car.Position = new Vector3( 10*i, 10*j, k * 10);
                        car.AngularVelocity = 0.1f;
                        car.AngleAxis = new Vector3(r.Next(10), r.Next(10), r.Next(10));
                        //car.Scale = new Vector3(5.0f*((i%3)+1), 5.0f*((j%3)+1), 5.0f*((k%3)+1));
                        car.Scale = new Vector3(5.0f , 5.0f , 5.0f );
                        car.SetGeometry(md2.vertices, md2.faces, SceneDevice.GetDevice(), md2.PositionFace);
                        car.Velocity = new Vector3(r.Next(35), r.Next(35), r.Next(35));
                        AddCollisionObj(car);
                    }
                }
            }

            //for (int i = 0; i < 3; i++)
            //{
            //    for (int j = 0; j < 3; j++)
            //    {
            //        for (int k = 0; k < 3; k++)
            //        {
            //            ZD3DObject car = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Blue.jpg", "glazed_brick_N.jpg", TextureMgr);
            //            car.Position = new Vector3(-i * 10, -j * 10, -k * 20);
            //            car.AngularVelocity = 0.1f;
            //            car.AngleAxis = new Vector3(r.Next(10), r.Next(10), r.Next(10));
            //            car.Scale = new Vector3(4.0f, 4.0f, 4.0f);
            //            car.SetGeometry(bc.vertices, bc.faces, SceneDevice.GetDevice(), bc.PositionFace);
            //            car.Velocity = new Vector3(r.Next(20), r.Next(20), r.Next(20));
            //            AddCollisionObj(car);
            //        }
            //    }
            //}









            Ground = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "glazed_brick_N.jpg", TextureMgr);
            Ground.Position = new Vector3(0.0f, 0.0f, -50.0f);
            Ground.Scale = new Vector3(100.0f, 100.0f, 1.0f);
            Ground.createCube(SceneDevice.GetDevice());

            Ground1 = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "glazed_brick_N.jpg", TextureMgr);
            Ground1.Position = new Vector3(-50.0f, 0.0f, 0.0f);
            Ground1.Scale = new Vector3(1.0f, 100.0f, 100.0f);
            Ground1.createCube(SceneDevice.GetDevice());


            Ground2 = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "glazed_brick_N.jpg", TextureMgr);
            Ground2.Position = new Vector3(50.0f, 0.0f, 0.0f);
            Ground2.Scale = new Vector3(1.0f, 100.0f, 100.0f);
            Ground2.createCube(SceneDevice.GetDevice());


            Ground3 = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "glazed_brick_N.jpg", TextureMgr);
            Ground3.Position = new Vector3(0.0f, -50, 0.0f);
            Ground3.Scale = new Vector3(100.0f, 1.0f, 100.0f);
            Ground3.createCube(SceneDevice.GetDevice());


            Ground4 = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "glazed_brick_N.jpg", TextureMgr);
            Ground4.Position = new Vector3(0.0f, 50.0f, 0.0f);
            Ground4.Scale = new Vector3(100.0f, 1.0f, 100.0f);
            Ground4.createCube(SceneDevice.GetDevice());




           // Gunlist = new List<ZD3DObject>();
           // SpringList = new List<ZD3DObject>();
           // Bullet = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "bump_N_round.jpg", TextureMgr);
           // Bullet.Mass = 1.0f;
           //// Bullet.AddConstraint(new AirFriction(Bullet.Mass/50.0f));
           // Bullet.Position = new Vector3(20 * (float)Math.Cos(0.1f), 0.0f, -20 * (float)Math.Sin(0.1f));
           // float value = 10000.0f * 0.02f / Bullet.Mass;
           // Bullet.Velocity = new Vector3(value * (float)Math.Cos(0.1f), 0.0f, -value * (float)Math.Sin(0.1f));
            
           // Bullet.Scale = new Vector3(3.0f, 3.0f, 3.0f);

           // Base = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "glazed_brick_D.jpg", "bump_N_round.jpg", TextureMgr);
           // Base.Position=new Vector3(0.0f, 0.0f, -120.0f);
           // Base.Scale = new Vector3(3.0f, 3.0f, 3.0f);

           // SpringMass = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "bump_N_round.jpg", TextureMgr);

           // SpringConstraint = new Spring(SpringMass.Mass, 15.0f, 1.0f, 10.0f, Base.Position);
           // SpringMass.AddConstraint(SpringConstraint);

           // SpringMass.Position = new Vector3(-20.0f, 0.0f, -120.0f);
           // SpringMass.Scale = new Vector3(3.0f, 3.0f, 3.0f);
           // SpringMass.Mass = 5;


           // ResetObjects();
           // for (int i = 0; i < 9; i++)
           // {
           //     ZD3DObject t = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "bump_N_round.jpg", TextureMgr);
           //     float angle;
           //     Vector3 angleAxis;
           //     angle = ((float)i / 9) * (2.0f * 3.14f);
           //     double k = Math.Cos(3.14);
           //     t.Position = new Vector3(0.0f,5.0f * (float)Math.Cos(angle),  5.0f * (float)Math.Sin(angle));
           //     angleAxis = new Vector3(1.0f, 0.0f, 0.0f);
           //     t.SetRotation(angle, angleAxis);
           //     t.MultiplyRotation(0.1f,new Vector3(0.0f,1.0f,0.0f));
           //     t.Scale = new Vector3(20.0f,1.0f , 2.0f * 2 * 2.0f * 3.14f / 9);
           //     Gunlist.Add(t);
           // }
           // for (int i = 0; i < 9; i++)
           // {
           //     Vector3 distance = Base.Position - SpringMass.Position;
           //     ZD3DObject t = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "bump_N_round.jpg", TextureMgr);
           //     float angle;
           //    // Vector3 angleAxis;
           //   //  angle = ((float)i / 9) * (2.0f * 3.14f);

           //     t.Position = new Vector3(distance.X * (float)i / 9, distance.Y * (float)i / 9, distance.Z * (float)i / 9);

           //     distance.Normalize();
           //    // angleAxis = distance;
           //     //t.SetRotation(angle, angleAxis);
                
           //     t.Scale = new Vector3(0.5f,0.5f,0.5f);
           //     SpringList.Add(t);
           // }
            
           // //at.worldPosition = new Vector3(0.0f, 0.0f, 0.0f);
            

           // a = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "bump_N_round.jpg", TextureMgr);
           // a.createCube(SceneDevice.GetDevice());

            //b = new ZD3DObject(SceneDevice.GetDevice(), SceneShader, "Wood.jpg", "bump_N_round.jpg", TextureMgr);

            broadphase = new Broadphase(CollisionObj);
            //broadphase.Update(CollisionObj);
            //b.createCube(SceneDevice.GetDevice());
          
        }
        public void ResetObjects()
        {
            Bullet.Mass = Mass;

            Bullet.Position = new Vector3(20 * (float)Math.Cos(Angle / 3.14), 0.0f, -20 * (float)Math.Sin(Angle / 3.14));
            float value = Powder * 0.02f / Bullet.Mass;
            float a = -value * (float)Math.Sin(Angle);
            Bullet.Velocity = new Vector3(value * (float)Math.Cos(Angle / 3.14), 0.0f, -value * (float)Math.Sin(Angle / 3.14));


            

            Bullet.Scale = new Vector3(3.0f, 3.0f, 3.0f);

            SpringMass.Position = new Vector3(-60.0f, 0.0f, -20.0f);
            SpringMass.Velocity = new Vector3(0.0f, 0.0f, 0.0f);
            SpringMass.Scale = new Vector3(3.0f, 3.0f, 3.0f);
            SpringMass.Accelerate = new Vector3(0.0f,0.0f,0.0f);


        }
      
        public Camera GetCamera()
        {
            return SceneCamera;
        }
        public void AddObject(Object obj)
        {
            SceneObjSets.AddObject(obj);
        
            obj.SceneMgr = this;
        }
        public void DeleteObject(Object obj)
        {
           
            if (obj != null && obj.IsDisposed == false)
            {
                obj.Dispose(SceneWorld);
                SceneObjSets.DeleteObject(obj);
            }
        }
        public void Render(float dt)
        {
            OnRendering = true;
           
            MoveView();

            
            //Begin the scene 
            SceneDevice.BeginScene(SceneShader.GetEffect());
            
            SceneCamera.UpdateCamera(); 
          

           // bunny2.Render(SceneShader, SceneDevice.GetDevice(), lighting, SceneCamera);

            for (int i = 0; i < CollisionObj.Count; i++)
            {
                CollisionObj[i].Render(SceneShader, SceneDevice.GetDevice(), lighting, SceneCamera);
            }
               


                //Ground.UpdateVertices(SceneDevice.GetDevice());
            Ground.Render(SceneShader, SceneDevice.GetDevice(), lighting, SceneCamera);
            Ground1.Render(SceneShader, SceneDevice.GetDevice(), lighting, SceneCamera);
         
            Ground3.Render(SceneShader, SceneDevice.GetDevice(), lighting, SceneCamera);
         

           
            SceneDevice.UnBindMap(SceneShader.GetEffect());

          

            SceneDevice.EndScene(); 
            
            
          
            DateTime prev = System.DateTime.Now;
            Confine();
            broadphase.Update(CollisionObj);
             //CollisionDetection();
            
            
            //int a =d.Second*1000+ d.Millisecond;
            
             BroadPhaseCollisionDetection();
             
            // DateTime c = after.Millisecond - after.Millisecond;

            // int b = d.Second * 1000+d.Millisecond;
           
             broadphase.Clear(CollisionObj.Count);
             DateTime after = System.DateTime.Now;
             TimeSpan t = after.Subtract(prev);

             Time += t.Milliseconds;
             FrameCount++;

             float average = Time / FrameCount;
            //PreviousTime=System.DateTime.Now; 
            OnRendering = false;
         }
        public D3DDevice GetDevice()
        {
            return SceneDevice;
        }
        public void BroadPhaseCollisionDetection()
        {
            for (int i = 0; i < broadphase.indexInv.Length; i++)
            {
                for (int j = 0; j < broadphase.indexInv[i].Count;j++)
                {
                    CollisionPairIndices collisionpar = new CollisionPairIndices(i, broadphase.indexInv[i][j], CollisionObj[i], CollisionObj[broadphase.indexInv[i][j]]);
                    if (gjk.Intersects(collisionpar.SupportFunction))
                    {

                        Vector3 distance = collisionpar.p1 - collisionpar.p2;

                        Vector3 normal = collisionpar.a.WorldVertics[collisionpar.a.indexSplit].normal;
                        normal.Normalize();
                        distance = Vector3.Dot(distance, normal) * normal * 0.5f;

                        collisionpar.a.Position += distance;
                        collisionpar.b.Position -= distance;
                        collisionpar.a.Velocity = -collisionpar.a.Velocity;
                        collisionpar.a.AngularVelocity = -collisionpar.a.AngularVelocity;
                        collisionpar.b.Velocity = -collisionpar.b.Velocity;
                        collisionpar.b.AngularVelocity = -collisionpar.b.AngularVelocity;


                        //collisionpar.a.Velocity = new Vector3(0.0f,0.0f,0.0f);
                        //collisionpar.a.AngularVelocity = 0.0f;
                        //collisionpar.b.Velocity = new Vector3(0.0f, 0.0f, 0.0f);
                        //collisionpar.b.AngularVelocity = 0.0f;
                    }
                }
                
            }

           
        }
        public bool Pick(Object obj)
        {
            Vector3 PickedPosition = new Vector3();
            for (int i = SceneObjSets.GetObjectCount()-1; i >=0; i--)
            {
                if (SceneObjSets.GetObjectByIndex(i).Pick(MousePosition[0],
                    MousePosition[1], out PickedPosition, SceneCamera))
                {
                    if (SceneObjSets.GetObjectByIndex(i) != obj)
                    {
                        return false;
                    }
                    else
                    {
                        PickingObj = obj;
                        PickingPosition = PickedPosition;
                        return true;
                    }
                }

            }
            return false;
                // obj.Pick(MousePosition[0], MousePosition[1], out PickedPosition, SceneCamera);
        }
        public void HandleInput(bool isMouseJointUsed)
        {
            //Use Middle Button to Countrol Zoom in or out
            if (MousePosition == null)
            {
                MousePosition = new int[2];
                MousePosition[0]=new int();
                MousePosition[0] = 0;
                MousePosition[1] = new int();
                MousePosition[1] = 0;
            }

            Pressed = false;
            Vector3 PickedPosition = new Vector3();
            
            for (int i = SceneObjSets.GetObjectCount() - 1; i >= 0; i--)
            {

                if (SceneObjSets.GetObjectByIndex(i).IsPhysics)
                    SceneObjSets.GetObjectByIndex(i).MouseJoint.Enabled = false;
                // SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Released;
                // SceneObjSets.GetObjectByIndex(i).Picked = false;
                if (SceneObjSets.GetObjectByIndex(i).Pick(MousePosition[0], MousePosition[1], out PickedPosition, SceneCamera))
                {
                    //SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Moved;
                    if (SceneDevice.GetMouseState().IsPressed(0))
                    {
                        Pressed = true;
                        if (SceneObjSets.GetObjectByIndex(i).Status == ObjectUIStatus.Released ||
                            SceneObjSets.GetObjectByIndex(i).Status == ObjectUIStatus.Moved)
                        {
                            SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Touched;
                        }
                        if (SceneObjSets.GetObjectByIndex(i).Status == ObjectUIStatus.Pressed)
                        {
                            SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Pressed;
                        }
                        if (SceneObjSets.GetObjectByIndex(i).IsPhysics)
                        {
                            if (SceneObjSets.GetObjectByIndex(i).Status == ObjectUIStatus.Touched)
                            {
                                Vector3 relPosition;
                                relPosition = PickedPosition - SceneObjSets.GetObjectByIndex(i).GetPosition();
                                SceneObjSets.GetObjectByIndex(i).RelToCenter = new Vector2(relPosition.X, relPosition.Y);

                                //Switch the status to Pressed

                                SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Pressed;
                            }


                            Vector2 ActualPos = new Vector2(PickedPosition.X, PickedPosition.Y) - SceneObjSets.GetObjectByIndex(i).RelToCenter;
                            if (isMouseJointUsed)
                            {
                                SceneObjSets.GetObjectByIndex(i).MouseJoint.Enabled = true;
                                SceneObjSets.GetObjectByIndex(i).MouseJoint.WorldAnchorB = new Vec2(ActualPos.X, ActualPos.Y);
                            }
                            else
                            {
                                SceneObjSets.GetObjectByIndex(i).GetBody().SetTransform(new Vec2(ActualPos.X, ActualPos.Y), 0.0f);
                            }
                        }

                    }
                    else
                    {
                        SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Moved;
                    }
                }
                else
                {
                    SceneObjSets.GetObjectByIndex(i).Status = ObjectUIStatus.Released;
                }
            }  
          
    
        }
        public void MoveView()
        {
            //Vector3 PickedPosition;
            //if (SceneDevice.GetMouseState().IsPressed(2))
            //{
            //    SceneCamera.Position += SceneCamera.LookAt * 0.1f - SceneCamera.Position * 0.1f;
            //}
            //Vector3 dir = SceneCamera.LookAt;
            //dir.Normalize();
            //SceneCamera.Position+=dir*(float)(SceneDevice.GetMouseState().Z);
            //USe Keyboard to control moving view 






            if ((SceneDevice.keyboradState.IsPressed(SlimDX.DirectInput.Key.A)))
            {

                SceneCamera.Rotation(0.1f, 0.0f, 0.0f);
            }

            if ((SceneDevice.keyboradState.IsPressed(SlimDX.DirectInput.Key.D)))
            {

                SceneCamera.Rotation(-0.1f, 0.0f, 0.0f);
            }

            if ((SceneDevice.keyboradState.IsPressed(SlimDX.DirectInput.Key.W)))
            {


                SceneCamera.Rotation(0.0f, 0.1f, 0.0f);
            }

            if ((SceneDevice.keyboradState.IsPressed(SlimDX.DirectInput.Key.S)))
            {


                SceneCamera.Rotation(0.0f, -0.1f, 0.0f);
            }

            if ((SceneDevice.keyboradState.IsPressed(SlimDX.DirectInput.Key.Q)))
            {


                SceneCamera.MoveForward(6.0f);
            }
            if ((SceneDevice.keyboradState.IsPressed(SlimDX.DirectInput.Key.E)))
            {


                SceneCamera.MoveForward(-6.0f);
            }
            

        }
        public void InitializeCircle(CircleObject circle, InitializeInf objInf, float radius)
        {
            circle.InitializeParameters(SceneDevice.GetDevice(), SceneShader, SceneWorld, objInf, radius, TextureMgr);
        }
       
        //Handle the hit of all objects
        public void CalculateHit(Object obj)
        {
            Vec2 relVelocity;
            float Harm=0.0f;
            for (int i = 0; i < SceneWorld.ContactManager .ContactList.Count; i++)
            {
                if (SceneWorld.ContactManager.ContactList[i].FixtureA.Body == obj.GetBody()||
                SceneWorld.ContactManager.ContactList[i].FixtureB.Body == obj.GetBody())
                {
                    
                    relVelocity=SceneWorld.ContactManager.ContactList[i].FixtureA.Body.LinearVelocity
                        -SceneWorld.ContactManager.ContactList[i].FixtureB.Body.LinearVelocity;
                    Vec2 localNormal=SceneWorld.ContactManager.ContactList[i].Manifold.LocalNormal;
                    //Harm += Math.Abs(localNormal.X * relVelocity.X + localNormal.Y * relVelocity.Y);
                    Harm += Math.Abs(Vec2.Dot(localNormal, relVelocity));  
                }
            }
            if (obj.Petrified)
            {
                obj.Hit = Harm;
            }
            else
            {
                obj.Hit = Harm;
            }
            obj.TotalHitBurden -= Harm;

            if (obj.Petrified)
            {
                obj.TotalHitBurden -= 50;
            }
        }  
        
    }
}
